function [tt,xx,A,B,Cn,On] = vizeses_simsolver(params, x0, tu, Ts)
%%
%  File: vizeses_simsolver.m
%  Directory: 4_gyujtemegy/11_CCS/_1_ccs/ccs_2020/vizeses
%  Author: Peter Polcz (ppolcz@gmail.com) 
%  
%  Created on 2020. September 24. (2020b)
%

Colors = {
    pcz_get_plot_colors([],2)
    pcz_get_plot_colors([],1)
    };

%%


params = num2cell(params);
[k01,k02,k11,k12,k2,k3] = deal(params{:});

k1 = k11 + k12;

A = [
   -k1   0   0
    k11 -k2  0
    k12  k2 -k3
    ];
A2 = A^2;


B = [
    k01
    k02
    0
    ];

C = [ 0 0 1 ];

Cn = ctrb(A,B);
On = obsv(A,C);

T = tu(end,1);
u = @(t) interp1(tu(:,1),tu(:,2),t);
f_ode = @(t,x) A*x + B*u(t);

[tt,xx] = ode45(f_ode, [0 T], x0);

if nargin < 4
    Ts = 0.1;
end
tt_linear = 0:Ts:T;
xx_linear = interp1(tt,xx,tt_linear);


%%

UPPER = max(max(xx));
LOWER = min(min(xx));

fig = figure(223);
delete(findall(fig,'type','annotation'))
delete(fig.Children)


annotation('arrow',[ 0.3 0.5 ],[0.73 0.65])
annotation('arrow',[ 0.3 0.2 ],[0.73 0.5])

annotation('arrow',[ 0.7 0.5 ],[0.5 0.35])
annotation('arrow',[ 0.7 0.8 ],[0.5 0.26])

annotation('arrow',[ 0.3 0.5 ],[0.26 0.12])


dim = [
    .4 .56 .1 .1
    .2 .56 .1 .1
    .55 .36 .1 .1
    .75 .32 .1 .1
    .34 .1 .1 .1
    .7 .0 .1 .03
    ];
for i = 1:6
    T1 = annotation('textbox',dim(i,:),'String',num2str(params{i}),'FitBoxToText','on');
    T1.LineStyle = 'none';
end

N = 4;
M = 2;
spl = [2 3 6]+2;

Tl = tiledlayout(N,M);
Tl.Padding = 'compact';
Tl.TileSpacing = 'none';

ax0 = nexttile(1,[1 2]);
Pl_xx = plot(tt,xx); 
hold on
grid on
Pl_u = plot(tt,u(tt),'k','LineWidth',2);
Pl0 = plot([0 0],[UPPER LOWER],'LineWidth',3,'Color',pcz_get_plot_colors([],5));

for i = 1:3
    ax(i) = nexttile(spl(i));
    ax(i).XLim = [0 1];
    ax(i).YLim = [LOWER UPPER];
    ax(i).XTick = ax(i).XLim;
    ax(i).XTickLabel = [];
    hold on
    box on 
    grid on
    Pl(i) = area([0 1],[1 1]*x0(i),'FaceColor',pcz_get_plot_colors([],1));
end

for k = 1:numel(tt_linear)
    Pl0.XData = [1 1]*tt_linear(k);
    for i = 1:3
        Pl(i).YData = [1 1]*xx_linear(k,i);
        Pl(i).FaceColor = Colors{1+double(xx_linear(k,i)>=0)};
    end
    drawnow
    pause(0.1)
end

end